# Generated by Django 2.2.16 on 2020-10-07 19:07

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_trips.mixins


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=70)),
                ('slug', models.SlugField(blank=True, max_length=85, null=True)),
                ('deleted', models.BooleanField(default=False)),
            ],
            bases=(django_trips.mixins.SlugMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Facility',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=70)),
                ('slug', models.SlugField(blank=True, max_length=85, null=True)),
                ('deleted', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name_plural': 'Facilities',
                'ordering': ['name'],
            },
            bases=(django_trips.mixins.SlugMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Host',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('slug', models.SlugField(blank=True, max_length=70, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('cancellation_policy', models.TextField(blank=True, null=True)),
                ('verified', models.BooleanField(default=False)),
            ],
            options={
                'ordering': ['name', 'verified'],
            },
            bases=(django_trips.mixins.SlugMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Location',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30)),
                ('slug', models.SlugField(blank=True, null=True)),
                ('coordinates', models.CharField(blank=True, max_length=50, null=True)),
                ('is_active', models.BooleanField(default=True)),
            ],
            options={
                'ordering': ['name'],
            },
            bases=(django_trips.mixins.SlugMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Trip',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500, verbose_name='Title')),
                ('slug', models.SlugField(blank=True, max_length=100, null=True)),
                ('description', models.TextField()),
                ('_metadata', models.TextField(blank=True, default='{}', null=True)),
                ('duration', models.SmallIntegerField(blank=True, default=0, null=True)),
                ('age_limit', models.SmallIntegerField(blank=True, default=0, null=True)),
                ('gear', models.TextField(blank=True, null=True, verbose_name='Recommended Gear')),
                ('deleted', models.BooleanField(default=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('category', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='django_trips.Category')),
                ('created_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='created_by_trips', to=settings.AUTH_USER_MODEL)),
                ('departure', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='trip_departure', to='django_trips.Location')),
                ('destination', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='trip_destination', to='django_trips.Location')),
                ('facilities', models.ManyToManyField(related_name='trip_facilities', to='django_trips.Facility')),
                ('host', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='host_trips', to='django_trips.Host')),
                ('locations', models.ManyToManyField(related_name='trip_locations', to='django_trips.Location')),
            ],
            options={
                'ordering': ['-created_at', '-id'],
            },
            bases=(django_trips.mixins.SlugMixin, models.Model),
        ),
        migrations.CreateModel(
            name='TripSchedule',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('price', models.SmallIntegerField(default=0)),
                ('date_from', models.DateTimeField()),
                ('trip', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='trip_schedule', to='django_trips.Trip')),
            ],
        ),
        migrations.CreateModel(
            name='TripReviewSummary',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('meals', models.FloatField(default=0)),
                ('accommodation', models.FloatField(default=0)),
                ('transport', models.FloatField(default=0)),
                ('value_for_money', models.FloatField(default=0)),
                ('overall', models.FloatField(default=0)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('trip', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='trip_review_summary', to='django_trips.Trip')),
            ],
            options={
                'verbose_name_plural': 'Trip review summaries',
            },
        ),
        migrations.CreateModel(
            name='TripReview',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('meals', models.SmallIntegerField(default=0)),
                ('accommodation', models.SmallIntegerField(default=0)),
                ('transport', models.SmallIntegerField(default=0)),
                ('value_for_money', models.SmallIntegerField(default=0)),
                ('overall', models.SmallIntegerField(default=0)),
                ('comment', models.TextField()),
                ('name', models.CharField(max_length=50)),
                ('email', models.EmailField(max_length=254)),
                ('is_verified', models.BooleanField(default=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('trip', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='trip_reviews', to='django_trips.Trip')),
            ],
        ),
        migrations.CreateModel(
            name='TripPickupLocation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('additional_price', models.SmallIntegerField(default=0)),
                ('location', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_trips.Location')),
                ('trip', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='trip_pickup_locations', to='django_trips.TripSchedule')),
            ],
        ),
        migrations.CreateModel(
            name='TripBooking',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=60)),
                ('phone_number', models.CharField(max_length=30)),
                ('cnic_number', models.CharField(max_length=30)),
                ('email', models.EmailField(max_length=254)),
                ('message', models.TextField()),
                ('schedule', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='trip_bookings', to='django_trips.TripSchedule')),
            ],
        ),
        migrations.CreateModel(
            name='CancellationPolicy',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('change_date', models.DateTimeField(auto_now_add=True, verbose_name='Change date')),
                ('enabled', models.BooleanField(default=False, verbose_name='Enabled')),
                ('description', models.TextField()),
                ('changed_by', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='Changed by')),
            ],
            options={
                'ordering': ('-change_date',),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TripItinerary',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('day', models.SmallIntegerField(default=0)),
                ('heading', models.CharField(blank=True, max_length=150, null=True)),
                ('description', models.TextField(default='')),
                ('trip', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='trip_itinerary', to='django_trips.Trip')),
            ],
            options={
                'verbose_name_plural': 'Trip Itineraries',
                'ordering': ['trip', 'day'],
                'unique_together': {('trip', 'day')},
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin

from django_trips.models import (
    Facility, Host, Location, Trip, TripItinerary,
    TripSchedule, CancellationPolicy, TripReview, TripReviewSummary,
    TripBooking, TripPickupLocation
)


class TripScheduleAdminInline(admin.TabularInline):
    """Trip schedule inline modal admin"""
    model = TripSchedule
    extra = 0


class TripReviewSummaryInline(admin.StackedInline):
    model = TripReviewSummary
    extra = 0


class TripItineraryAdminInline(admin.StackedInline):
    """Trip itinerary inline modal admin"""
    model = TripItinerary
    extra = 0


admin.register(CancellationPolicy)


@admin.register(Trip)
class TripAdmin(admin.ModelAdmin):
    """Trip modal admin configuration"""
    inlines = [TripItineraryAdminInline, TripScheduleAdminInline, TripReviewSummaryInline]
    prepopulated_fields = {'slug': ('name',)}
    list_display = ('name', 'slug',)
    list_filter = ('destination',)
    search_fields = ['name', 'description', 'slug', 'locations__name']


@admin.register(Facility)
class FacilityAdmin(admin.ModelAdmin):
    """Facility modal admin configuration"""
    prepopulated_fields = {'slug': ('name',)}
    list_display = (
        'name', 'slug',
    )


@admin.register(Location)
class LocationAdmin(admin.ModelAdmin):
    """Location modal admin configuration"""
    prepopulated_fields = {'slug': ('name',)}
    list_display = (
        'name', 'slug',
    )


@admin.register(TripItinerary)
class TripItineraryAdmin(admin.ModelAdmin):
    """Trip itinerary modal admin configuration"""
    list_display = ('trip', 'description')
    list_filter = ('trip',)
    search_fields = ['trip']


@admin.register(TripSchedule)
class TripScheduleAdmin(admin.ModelAdmin):
    """Trip schedule admin configuration"""
    list_display = ('trip', 'date_from')
    list_filter = ('date_from',)
    search_fields = ['trip']


@admin.register(Host)
class HostAdmin(admin.ModelAdmin):
    """Host modal admin configuration"""
    list_display = (
        'name', 'description', 'verified'
    )
    list_filter = ('verified',)
    search_fields = ['name', 'description']
    prepopulated_fields = {'slug': ('name',)}


@admin.register(TripReview)
class TripReviewAdmin(admin.ModelAdmin):
    list_display = (
        'name', 'trip', 'meals', 'accommodation', 'transport',
        'value_for_money', 'overall',
    )
    list_filter = ('is_verified', 'trip', 'overall')
    search_fields = ['trip__name', 'name']


@admin.register(TripReviewSummary)
class TripReviewSummaryAdmin(admin.ModelAdmin):
    # inlines = (TripInline,)
    list_display = (
        'trip', 'meals', 'accommodation', 'transport',
        'value_for_money', 'overall',
    )
    list_filter = ('overall',)
    search_fields = ['trip__name']


@admin.register(TripPickupLocation)
class TripPickupLocationAdmin(admin.ModelAdmin):
    list_display = (
        'trip', 'location', 'additional_price'
    )
    search_fields = ['trip__name']
    list_filter = ('location',)


@admin.register(TripBooking)
class TripBookingSummaryAdmin(admin.ModelAdmin):
    list_display = ('name', 'schedule', 'phone_number', 'message')
    search_fields = ['schedule__trip__name', 'name']
    list_filter = ('schedule__trip__name', 'schedule__date_from')



import django_filters
from django_trips.models import Trip


class TripFilter(django_filters.FilterSet):
    name = django_filters.CharFilter(field_name='name', lookup_expr=('icontains'))
    duration = django_filters.NumberFilter(field_name='duration', lookup_expr=('gte'))
    price = django_filters.NumberFilter(field_name='trip_schedule__price', method='filter_price')
    destination = django_filters.CharFilter(field_name='destination', method='filter_price')

    from_date = django_filters.DateFilter(
        label="Trip From Date",
        field_name='trip_schedule__date_from',
        method='filter_from_date',
    )
    to_date = django_filters.DateFilter(
        label="Trip To Date",
        field_name='trip_schedule__date_from',
        method='filter_to_date',
    )

    def filter_from_date(self, queryset, name, value):
        lookup = '__'.join([name, 'gte'])
        return queryset.filter(**{lookup: value})

    def filter_to_date(self, queryset, name, value):
        lookup = '__'.join([name, 'lte'])
        return queryset.filter(**{lookup: value})

    def filter_price(self, queryset, name, value):
        lookup = '__'.join([name, 'gte'])
        return queryset.filter(**{lookup: value})

    class Meta:
        model = Trip
        fields = ('name', 'price')



# -*- coding: utf-8 -*-

import random
from datetime import datetime, timedelta

from dateutil.tz import UTC
from django.utils.text import slugify
from django.utils.timezone import make_aware
from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.models import User

from django_trips.models import (
    Facility, Host, Location, Trip, TripItinerary, Category, TripSchedule
)


class Command(BaseCommand):
    """
    This command will generate batch of trips with random data.

    By running this command, the trips generated will have random data
    from a pre-defined list of data. The data selection is done at the
    random to ensure that trips will have diverse data, which will provide
    help for filtering & searching.

    EXAMPLE USAGE:
        ./manage.py generate_trips --batch_size=100
    OR
        ./manage.py generate_trips

    If batch size is not provided, the command will generate 10 trips

    """

    help = "Generate batches of trip with pre-populated random data"
    locations_names_list = [
        'Fairy Meadows', 'Hunza', 'Gilgit', 'Lahore', 'Islamabad',
        'Karachi', 'Kashmir', 'Murree', 'Kaghan', 'Swat', 'Skardu'
    ]

    def add_arguments(self, parser):
        """
        Defining the arguments to be used by the command.
        """
        parser.add_argument(
            '--batch_size',
            type=int,
            default=10,
            dest='batch_size',
            help="number to trips to generate"
        )

    @staticmethod
    def get_random_host():
        """
        Get a random host from a pre-defined list of hosts
        """
        host_list = [
            'Arbisoft', 'Traverse', 'Travel Freaks', 'Destivels', 'Arbitainment'
        ]
        host, __ = Host.objects.get_or_create(name=random.choice(host_list), verified=True)
        return host

    def get_location_instance(self, name=None):
        name = name or random.choice(self.locations_names_list)
        location_data = {'name': name, 'slug': slugify(name)}
        return location_data

    def get_destination(self):
        location_data = self.get_location_instance()
        destination, __ = Location.objects.get_or_create(slug=location_data['slug'], defaults=location_data)
        return destination

    def get_random_locations(self):
        """
        Get multiple/single location object from pre-defined list.
        """
        trip_random_locations = []
        for location_name in random.sample(self.locations_names_list, random.choice(range(2, 7))):
            location_data = self.get_location_instance(location_name)
            location, __ = Location.objects.get_or_create(slug=location_data['slug'], defaults=location_data)
            trip_random_locations.append(location)
        return trip_random_locations

    @staticmethod
    def get_random_facilities():
        """
        Get random number of facilities from some pre-defined facilities.
        """
        facilities_names_list = [
            'Transport', 'Meals', 'Guide', 'Photography', 'Accommodation',
            'First Aid Kit', 'Bon Fire', 'Power Bank'
        ]
        facilities_objects_list = []
        for facility_name in random.sample(facilities_names_list, random.choice(range(2, 5))):
            facility, __ = Facility.objects.get_or_create(
                name=facility_name,
                defaults={'slug': slugify(facility_name)}
            )
            facilities_objects_list.append(facility)
        return facilities_objects_list

    @staticmethod
    def get_random_category():
        """
        Get random number of facilities from some pre-defined facilities.
        """
        category_names_list = [
            'Long Drive', 'Honeymoon', 'Rode Trip', 'Bone fire', 'Hiking',
        ]
        name = random.choice(category_names_list)
        category, __ = Category.objects.get_or_create(
            name=name,
            defaults={'slug': slugify(name)}
        )
        return category

    @staticmethod
    def get_random_schedules(trip_duration):
        """
        get random schedules datetime objects separated by appropriate trip duration.
        """
        schedules_list = []
        for data_range in range(1, random.choice(range(3, 6))):
            schedule = datetime.now(tz=UTC) + timedelta(days=trip_duration + (data_range * 7))
            schedules_list.append(schedule)
        return schedules_list

    @staticmethod
    def get_random_itineraries():
        """
        generate random itineraries for a trip with a defined format
        """
        return [
            (day, "Itinerary for Day: {}".format(day))
            for day in range(1, random.choice(range(4, 7)))
        ]

    @staticmethod
    def get_random_gear():
        """
        get random number of gears for a trip.

        returns a comma-separated string of gears from a pre-defined
        list of gears
        """
        gears_list = [
            'Mountain Climber', 'Shoes', 'Stick', 'Coat', 'Camp',
            'Inhaler', 'Lighter'
        ]
        selected_gear = random.sample(gears_list, random.choice(range(1, 4)))
        return ','.join(selected_gear)

    def handle(self, *args, **options):
        """
        Generating trip based on the input.
        """
        batch_size = options['batch_size']
        # user required to create a trip
        super_users = User.objects.filter(is_superuser=True)
        if not super_users.count() > 0:
            raise CommandError("Superuser doesn't exist. Please create a superuser for the app.")
        user = super_users.first()

        for count in range(0, batch_size):
            trip = Trip(name="Trip : {}".format(count))
            trip_itineraries = self.get_random_itineraries()

            trip.destination = self.get_destination()
            trip.departure = self.get_destination()

            trip.name = "{} days trip to {}".format(len(trip_itineraries), trip.destination.name)

            trip.duration = random.choice(range(3, 8))
            trip.age_limit = random.choice(range(20, 40))
            trip.host = self.get_random_host()
            trip.gear = self.get_random_gear()
            trip.category = self.get_random_category()
            trip.description = "This is the description for trip: {}".format(count)
            trip.created_by = user

            # Initial Save
            try:
                trip.save()
            except Exception as e:
                raise CommandError(self.stderr.write(
                    'Error Saving Trip {}\n{}'.format(trip.name, e)))

            # Adding M2M fields for the trip
            trip.locations.set(self.get_random_locations())
            trip.facilities.set(self.get_random_facilities())
            trip.save()

            # Setting Schedule & Itinerary
            trip_schedules = self.get_random_schedules(trip.duration)
            price = random.choice([1000, 5000, 6000, 9000])
            for schedule in trip_schedules:
                price = random.choice([price - 500, price + 500])
                trip_schedule = TripSchedule(trip=trip, date_from=schedule, price=price)
                trip_schedule.save()

            for itinerary in trip_itineraries:
                trip_itinerary = TripItinerary(
                    trip=trip,
                    heading="Day {}".format(itinerary[0]),
                    description=itinerary[1],
                    day=itinerary[0],
                )
                trip_itinerary.save()

            self.stdout.write(self.style.SUCCESS('Trip Created: %s') % trip.name)



"""Urls for trips app"""
from django.urls import path

from django_trips.api import views

app_name = "trips-api"

urlpatterns = [
    path('trips/', views.TripListCreateAPIView.as_view(), name="trips"),
    path('trip/<int:pk>/', views.TripRetrieveUpdateDestroyAPIView.as_view(), name="trip-item"),
    path('trip/<slug:slug>/', views.TripRetrieveUpdateDestroyAPIView.as_view(), name="trip-item-slug"),
]



"""Test management command"""
from django.core.management import call_command
from django.test import TestCase
from six import StringIO

from django_trips.tests import factories


class CommandsTestBase(TestCase):
    """
    Command for creating random trips.
    """

    def setUp(self):
        self.superuser = factories.UserFactory.create(is_superuser=True)
        super().setUp()

    def run_generate_trips_command(self, *args, **kwargs):
        """
        Calls the test command and outputs a dict of the current context.
        """
        out = StringIO()
        call_command('generate_trips', *args, stdout=out, **kwargs)
        out.seek(0)
        return out.read().strip()

    def test_command(self):
        trip_count = 10
        kwargs = {'batch_size': trip_count}
        output = self.run_generate_trips_command(*(), **kwargs)
        self.assertEqual(output.count('Trip Created'), trip_count)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import generics
from rest_framework.authentication import SessionAuthentication, BasicAuthentication
from rest_framework.permissions import IsAuthenticated

from . import serializers
from .filters import TripFilter
from .mixins import MultipleFieldLookupMixin
from .paginators import TripResponsePagination
from django_trips.models import Trip


class TripListCreateAPIView(generics.ListCreateAPIView):
    """
    Trips List and Create Trip view.

    This endpoint is used to list all the trips Using the GET request type.
    This endpoint is used to create a single trip using the POST request type. Provide all required data in
    the request body.

    Examples:
        POST: Creates a new trip
            /api/trips/
            data = {
                "name":"3 days trip to Islamabad",
                "description": "This is the description for trip: 66",
                "locations": [1,2],
                "facilities": [1,2],
                "destination": 1,
                "created_by": 1,
                "host": 1,
            }

        GET: Return all trips (paginated, 10per page.)
            /api/trips/
        GET: Search Trips that contains specific name.
            /api/trips/?name=Islamabad
            Other Options.
            | Keyword       |                                                                       |
            | name          | Find trips that contains specific name.                               |
            | price         | Find trips that contains price greater than or equal                  |
            | duration      | Find trips of duration greater than or equal                          |
            | from_date     | Find trips that are scheduled greater than or specified date          |
            | to_date       | Find trips that are scheduled less than or equal to specified date    |

    """

    authentication_classes = [SessionAuthentication, BasicAuthentication]
    permission_classes = [IsAuthenticated]
    filter_backends = [DjangoFilterBackend]
    filter_class = TripFilter
    queryset = Trip.active.all()

    def get_serializer_class(self):
        if self.request.method == 'POST':
            return serializers.TripSerializerWithIDs
        return serializers.TripDetailSerializer


class TripRetrieveUpdateDestroyAPIView(
    MultipleFieldLookupMixin, generics.RetrieveUpdateDestroyAPIView
):
    """
    Trip retrieve/update/destroy API View.

    This endpoint is used to fetch a single trip using GET request type.
    This endpoint is used to update a single trip using PUT request type.
    This endpoint is used to partially update a single trip using PATCH request type.
    This endpoint is used to delete a single trip using DELETE request type.

    Examples:
        GET: Fetch Trip by ID/Slug
            api/trip/99/
            api/trip/5-days-trip-to-jhelum/

        PUT: Update the whole object.
            api/trip/99/
            data = {
                ...
                age_limit: 39
            }

        PATCH: Partially update object values
            data = {age_limit: 39}
    """

    pagination_class = TripResponsePagination
    authentication_classes = [SessionAuthentication, BasicAuthentication]
    permission_classes = [IsAuthenticated]
    queryset = Trip.active.all()

    lookup_fields = ['pk', 'slug']

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return serializers.TripDetailSerializer
        return serializers.TripSerializerWithIDs



"""Django Trips serializers"""
from django.contrib.auth.models import User
from django.urls import reverse
from rest_framework import serializers

from django_trips.models import (
    Facility,
    Host,
    Location,
    Trip,
    TripItinerary,
    TripSchedule,
    Category)


class UserSerializer(serializers.ModelSerializer):
    """User Modal Serializer"""

    class Meta:
        model = User
        fields = ('username', 'email', 'is_staff')


class CategorySerializer(serializers.ModelSerializer):
    """Category Serializer"""

    class Meta:
        model = Category
        fields = '__all__'


class LocationSerializer(serializers.ModelSerializer):
    """Location Modal Serializer"""

    class Meta:
        model = Location
        fields = '__all__'


class FacilitySerializer(serializers.ModelSerializer):
    """Facility Modal Serializer"""

    class Meta:
        model = Facility
        fields = '__all__'


class HostSerializer(serializers.ModelSerializer):
    """Host Modal Serializer"""

    class Meta:
        model = Host
        fields = '__all__'


class TripItinerarySerializer(serializers.ModelSerializer):
    """TripItinerary Modal Serializer"""

    class Meta:
        model = TripItinerary
        fields = "__all__"


class TripScheduleSerializer(serializers.ModelSerializer):
    """TripSchedule Modal Serializer"""

    class Meta:
        model = TripSchedule
        exclude = ('trip',)


class TripSerializerWithIDs(serializers.ModelSerializer):
    class Meta:
        model = Trip
        exclude = ('_metadata',)

    def create(self, validated_data):
        return super(TripSerializerWithIDs, self).create(validated_data)


class TripBaseSerializer(serializers.ModelSerializer):
    trip_url = serializers.SerializerMethodField()
    metadata = serializers.JSONField(source="_metadata", initial="{}")
    departure = LocationSerializer()
    destination = LocationSerializer()
    created_by = UserSerializer(read_only=True)
    trip_schedule = serializers.SerializerMethodField()
    trip_itinerary = TripItinerarySerializer(many=True)
    locations = LocationSerializer(many=True)
    facilities = FacilitySerializer(many=True)
    category = CategorySerializer()

    class Meta:
        fields = (
            'name',
            'slug',
            'trip_url',
            'description',
            'duration',
            'age_limit',
            'departure',
            'destination',
            'category',
            'gear',
            'metadata',
            'locations',
            'trip_schedule',
            'trip_itinerary',
            'facilities',
            'created_by',
        )
        model = Trip

    def get_trip_url(self, trip):
        return reverse("trips-api:trip-item-slug", kwargs={"slug": trip.slug})

    def get_trip_schedule(self, trip):
        qs = TripSchedule.available.filter(trip=trip)
        serializer = TripScheduleSerializer(instance=qs, many=True)
        return serializer.data


class TripDetailSerializer(TripBaseSerializer):
    """
    Trip Detail Modal Serializer

    This Serializer is used for trip object retrieval as it requires all
    the details of the object
    """

    cancellation_policy = serializers.CharField(read_only=True)
    host = HostSerializer(read_only=True)

    def get_cancellation_policy(self, trip):
        return trip.cancellation_policy

    class Meta:
        model = Trip
        fields = TripBaseSerializer.Meta.fields + ('cancellation_policy', 'host')



"""API Mixins"""
from rest_framework.generics import get_object_or_404


class MultipleFieldLookupMixin(object):
    """Mixin for using multi field lookups."""

    def get_object(self):
        queryset = self.filter_queryset(self.get_queryset())  # Apply any filter backends
        filter = {}
        for field in self.lookup_fields:
            if self.kwargs.get(field):
                filter[field] = self.kwargs[field]

        obj = get_object_or_404(queryset, **filter)  # Lookup the object
        self.check_object_permissions(self.request, obj)
        return obj



import json
from datetime import timezone, datetime

from django.db import models
from django.urls import reverse
from django.utils.text import slugify

import django_trips.managers as managers
from config_models.models import ConfigurationModel

from django_trips.mixins import SlugMixin


class Host(SlugMixin, models.Model):
    """
    Trip host model.

    This model contains the information for the trip hosts who are organizing
    trips.
    """
    name = models.CharField(max_length=50)
    slug = models.SlugField(max_length=70, null=True, blank=True)
    description = models.TextField(null=True, blank=True)
    cancellation_policy = models.TextField(null=True, blank=True)
    verified = models.BooleanField(default=False)

    class Meta:
        ordering = ['name', 'verified']

    def __str__(self):
        """String representation of model instance"""
        return self.name


class Location(SlugMixin, models.Model):
    """
    Trip location model

    This model contains information about trip location with respect to
    coordinates. We will use coordinates to draw google map.
    """
    objects = models.Manager()
    destinations = managers.TripDestinationManager()
    departures = managers.TripDepartureManager()
    available = managers.AvailableLocationManager()

    name = models.CharField(max_length=30)
    slug = models.SlugField(null=True, blank=True)
    coordinates = models.CharField(max_length=50, null=True, blank=True)
    is_active = models.BooleanField(default=True)

    class Meta:
        ordering = ['name']

    def __str__(self):
        """String representation of model instance"""
        return "{0}".format(self.name)

    @property
    def get_coordinates(self):
        """Returns lat/lng in list."""
        return self.coordinates.split(',')


class Facility(SlugMixin, models.Model):
    """
    Trip Facility model

    This model contains information all the available facilities that can be
    provided in a trip.
    """
    name = models.CharField(max_length=70)
    slug = models.SlugField(max_length=85, null=True, blank=True)
    deleted = models.BooleanField(default=False)

    class Meta:
        ordering = ['name']
        verbose_name_plural = 'Facilities'

    def __str__(self):
        """String representation of model instance"""
        return self.name


class Category(SlugMixin, models.Model):
    """Trip Category model"""
    name = models.CharField(max_length=70)
    slug = models.SlugField(max_length=85, null=True, blank=True)
    deleted = models.BooleanField(default=False)

    def __str__(self):
        return self.name


class Trip(SlugMixin, models.Model):
    """
    Trip model

    This model contains the main information that will be presented to
    end users.
    """
    objects = models.Manager()
    active = managers.ActiveTripManager()

    name = models.CharField("Title", max_length=500)
    slug = models.SlugField(max_length=100, null=True, blank=True)
    description = models.TextField()
    # meta includes tinyurl, poster
    _metadata = models.TextField(default='{}', null=True, blank=True)

    duration = models.SmallIntegerField(default=0, null=True, blank=True)
    age_limit = models.SmallIntegerField(default=0, null=True, blank=True)

    destination = models.ForeignKey(Location, related_name='trip_destination', on_delete=models.CASCADE)
    departure = models.ForeignKey(Location, related_name='trip_departure', on_delete=models.CASCADE)
    locations = models.ManyToManyField(Location, related_name="trip_locations")

    category = models.ForeignKey(Category, null=True, blank=True, on_delete=models.CASCADE)
    facilities = models.ManyToManyField(Facility, related_name="trip_facilities")
    gear = models.TextField("Recommended Gear", null=True, blank=True)

    deleted = models.BooleanField(default=False)
    created_by = models.ForeignKey(
        'auth.User', related_name="created_by_trips", on_delete=models.CASCADE
    )
    host = models.ForeignKey(Host, related_name="host_trips", on_delete=models.CASCADE)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        """String representation of model instance"""
        return self.name

    class Meta:
        ordering = ['-created_at', '-id']

    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)
        self.slug = slugify(self.name)

    def get_absolute_url(self):
        return reverse('view_trip', {'slug': self.slug})

    @property
    def metadata(self):
        """Parse the internal metadata field into python object"""
        return json.loads(self._metadata)

    @property
    def cancellation_policy(self):
        """
        Trip's cancellation policy should be given preference over the
        generic host cancellation (all-host-trips) policy.
        """
        return self.host.cancellation_policy or CancellationPolicy.current().description


class TripItinerary(models.Model):
    """
    Trip itinerary model

    This model describes a trip with respect to each day.
    """
    trip = models.ForeignKey(Trip, related_name="trip_itinerary", on_delete=models.CASCADE)
    day = models.SmallIntegerField(default=0)
    heading = models.CharField(max_length=150, null=True, blank=True)
    description = models.TextField(default='')

    def __str__(self):
        """String representation of model instance"""
        return "Day:{0}-{1}".format(self.day, self.trip.name)

    class Meta:
        ordering = ['trip', 'day']
        verbose_name_plural = 'Trip Itineraries'
        unique_together = ('trip', 'day',)


class TripSchedule(models.Model):
    """
    Trip schedule model

    This model contains information of upcoming trips
    """
    objects = models.Manager()
    available = managers.AvailableTripScheduleManager()

    trip = models.ForeignKey(Trip, related_name="trip_schedule", on_delete=models.CASCADE)
    price = models.SmallIntegerField(default=0)
    date_from = models.DateTimeField()

    def __str__(self):
        """String representation of model instance"""
        return self.trip.name

    @property
    def is_active(self):
        return self.date_from <= datetime.now()


class TripReview(models.Model):
    """Trip Review Model"""
    trip = models.ForeignKey(Trip, related_name="trip_reviews", on_delete=models.CASCADE)
    meals = models.SmallIntegerField(default=0)
    accommodation = models.SmallIntegerField(default=0)
    transport = models.SmallIntegerField(default=0)
    value_for_money = models.SmallIntegerField(default=0)
    overall = models.SmallIntegerField(default=0)
    comment = models.TextField()
    #     User details
    name = models.CharField(max_length=50)
    email = models.EmailField()
    is_verified = models.BooleanField(default=False)
    #     timestamps
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return "{}-{}".format(self.name, self.overall)


class TripReviewSummary(models.Model):
    """Trip Review Summary Model"""
    trip = models.OneToOneField(
        Trip, related_name="trip_review_summary", on_delete=models.CASCADE, null=True, blank=True
    )
    meals = models.FloatField(default=0)
    accommodation = models.FloatField(default=0)
    transport = models.FloatField(default=0)
    value_for_money = models.FloatField(default=0)
    overall = models.FloatField(default=0)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        verbose_name_plural = "Trip review summaries"


class CancellationPolicy(ConfigurationModel):
    """CancellationPolicyModel model"""
    description = models.TextField()

    def __str__(self):
        return self.description


class TripBooking(models.Model):
    """Trip booking"""
    schedule = models.ForeignKey(TripSchedule, related_name="trip_bookings", on_delete=models.CASCADE)
    name = models.CharField(max_length=60)
    phone_number = models.CharField(max_length=30)
    cnic_number = models.CharField(max_length=30)
    email = models.EmailField()
    message = models.TextField()

    def __str__(self):
        return self.name


class TripPickupLocation(models.Model):
    """Trip pickup locations"""
    trip = models.ForeignKey(TripSchedule, related_name="trip_pickup_locations", on_delete=models.CASCADE)
    location = models.ForeignKey(Location, on_delete=models.CASCADE)
    additional_price = models.SmallIntegerField(default=0)

    def __str__(self):
        return self.location



from datetime import datetime, timedelta

import ddt
from django.urls import reverse
from django.utils.timezone import get_current_timezone
from rest_framework.test import APITestCase

from django_trips.tests import factories


@ddt.ddt
class TestTripList(APITestCase):
    trips_list_url = reverse("trips-api:trips")

    def setUp(self):
        self.user = factories.UserFactory()
        self.client.login(username=self.user.username, password=factories.USER_PASSWORD)
        self.trip = factories.TripFactory(locations=['Lahore', 'Gilgit'])
        self.trip_schedules = factories.TripScheduleFactory.create_batch(2, trip=self.trip)

    def _make_trip_schedules_in_past(self, trip=None):
        """Convert trip schedules in past"""
        trip = trip if trip else self.trip
        today = datetime.now(tz=get_current_timezone())
        past_trip_date = today - timedelta(days=7)

        trip_schedules = trip.trip_schedule.all()
        trip_schedules.update(date_from=past_trip_date)

    def get_trips_list_result(self):
        """ Get trip list by calling api and return results"""
        response = self.client.get(self.trips_list_url, {})
        return response.json()['results']

    def test_without_authentication(self):
        """
        Verify that we have access to all the data w/o authentication.
        """
        self.client.logout()
        response = self.client.get(self.trips_list_url, {})
        self.assertEqual(response.status_code, 403)

    def test_trip_list_pagination(self):
        """
        Verify if the returned data has expected pagination keys
        """
        expected_keys = ['count', 'results', 'next', 'current', 'pages', 'previous']
        response = self.client.get(self.trips_list_url, {})
        pagination_keys = response.json()
        self.assertTrue(all([expected_key in pagination_keys for expected_key in expected_keys]))

    def test_trip_list_results(self):
        """Verify if the returned data contains expected result keys"""
        expected_keys = [
            'name', 'slug', 'description', 'duration', 'age_limit', 'destination', 'metadata', 'category',
            'gear', 'created_by', 'trip_schedule', 'trip_itinerary', 'locations', 'facilities',
            'trip_url', 'cancellation_policy', 'host'
        ]
        results = self.get_trips_list_result()
        self.assertEqual(len(results), 1)

        result_keys = results[0].keys()
        self.assertTrue(all([expected_key in result_keys for expected_key in expected_keys]))

    def test_filter_inactive_trips(self):
        """Verify inactive trips are not returned in api call"""
        trips = self.get_trips_list_result()
        self.assertEqual(len(trips), 1)
        self.trip.deleted = True
        self.trip.save()
        trips = self.get_trips_list_result()
        self.assertEqual(len(trips), 0)

    def test_filter_inactive_schedule_trips(self):
        """
        Verify that api filters out inactive trips

        Create a trip with active schedules and verify it is listed
        correctly.
        Mark the trip schedule dates in the past and try to fetch trip lists again.
        Verify the trip is filtered out.
        """
        trips = self.get_trips_list_result()
        self.assertEqual(len(trips), 1)
        trip_schedule_via_api = trips[0]['trip_schedule']

        trip_schedules = self.trip.trip_schedule.all()
        self.assertEqual(trip_schedules.count(), len(trip_schedule_via_api))

        self._make_trip_schedules_in_past()

        trips = self.get_trips_list_result()
        self.assertEqual(len(trips), 1)
        trip_schedule_via_api = trips[0]['trip_schedule']
        self.assertEqual(len(trip_schedule_via_api), 0)



from datetime import datetime

from django.db import models
from pytz import UTC


class AvailableTripScheduleManager(models.Manager):
    """
    Trip schedule safe queryset manager.

    Usage:
        >>> TripSchedule.available.all()
    """

    def get_queryset(self):
        """
        This method will only return the objects which have date_from defined
        in the future
        """
        return super(AvailableTripScheduleManager, self).get_queryset().filter(
            date_from__gt=datetime.now(tz=UTC)
        )


class AvailableLocationManager(models.Manager):
    """
    Trip schedule safe queryset manager.

    Usage:
        >>> Location.available.all()
    """

    def get_queryset(self):
        """
        This method will only return the objects which are active.
        """
        return super().get_queryset().filter(is_active=True)


class ActiveTripManager(models.Manager):
    """
    Trip schedule safe queryset manager.

    Usage:
        >>> Trip.active.all()
        >>> Trip.trip_schedule(manager='active').all()
    """

    def get_queryset(self):
        """
        This method will only return the objects which have date_from defined
        in the future
        """
        return super(ActiveTripManager, self).get_queryset().filter(
            deleted=False
        )


class TripDestinationManager(models.Manager):
    """
    Trip schedule safe queryset manager.

    Usage:
        >>> Location.destinations.all()
        >>> Trip.locations(manager='destinations').all()
    """

    def get_queryset(self):
        """
        This method will only return the objects which have date_from defined
        in the future
        """
        return super(TripDestinationManager, self).get_queryset().filter(
            is_destination=True
        )


class TripDepartureManager(models.Manager):
    """
    Trip schedule safe queryset manager.

    Usage:
        >>> Location.departures.all()
        >>> Trip.locations(manager='departures').all()
    """

    def get_queryset(self):
        """
        This method will only return the objects which have date_from defined
        in the future
        """
        return super(TripDepartureManager, self).get_queryset().filter(
            is_departure=False
        )



from datetime import datetime

import factory.fuzzy
from dateutil.relativedelta import relativedelta
from django.contrib.auth.models import Group, User
from factory.django import DjangoModelFactory
from pytz import UTC

from django_trips.models import (
    Category,
    Facility,
    Host,
    Location,
    Trip,
    TripItinerary,
    TripSchedule,
)

USER_PASSWORD = 'pswd'


class GroupFactory(DjangoModelFactory):
    """Group factory"""

    class Meta(object):
        model = Group
        django_get_or_create = ("name",)

    name = factory.Sequence("group{0}".format)


class UserFactory(DjangoModelFactory):
    """User factory"""

    username = factory.Sequence(u'User - {0}'.format)
    password = factory.PostGenerationMethodCall('set_password', USER_PASSWORD)

    class Meta:
        model = User


class CategoryFactory(DjangoModelFactory):
    """Category factory"""

    class Meta(object):
        model = Category

    name = factory.Sequence("Category - {0}".format)
    slug = factory.Sequence("category-{0}".format)


class HostFactory(DjangoModelFactory):
    """Host factory"""

    class Meta(object):
        model = Host

    name = factory.Sequence("Host - {0}".format)
    cancellation_policy = factory.Sequence("cancellation Policy - {0}".format)


class FacilityFactory(DjangoModelFactory):
    """Facility factory"""

    class Meta(object):
        model = Facility

    name = factory.Sequence("Facility - {0}".format)


class LocationFactory(DjangoModelFactory):
    """Location factory"""

    class Meta(object):
        model = Location

    name = factory.Sequence("Location - {0}".format)
    # is_destination = factory


class TripScheduleFactory(DjangoModelFactory):
    """TripSchedule factory"""

    class Meta:
        model = TripSchedule

    date_from = factory.fuzzy.FuzzyDateTime(
        datetime.now(tz=UTC), datetime.now(tz=UTC) + relativedelta(months=+1)
    )


class TripItineraryFactory(DjangoModelFactory):
    class Meta:
        model = TripItinerary


class TripFactory(DjangoModelFactory):
    """Trip factory"""

    class Meta(object):
        model = Trip
        django_get_or_create = ("slug",)

    name = factory.Sequence("My awsome trip - {0}".format)
    slug = factory.Sequence("my-awsome-trip-{0}".format)
    description = factory.Sequence("awsome trip description- {}".format)

    destination = factory.SubFactory(LocationFactory)
    departure = factory.SubFactory(LocationFactory)
    locations = factory.SubFactory(LocationFactory)
    category = factory.SubFactory(CategoryFactory)
    facilities = factory.SubFactory(FacilityFactory)
    trip_schedule = factory.SubFactory(TripScheduleFactory)

    gear = factory.Sequence("Trip gear - {0}".format)

    created_by = factory.SubFactory(UserFactory)
    host = factory.SubFactory(HostFactory)

    created_at = datetime(2012, 1, 1, tzinfo=UTC)
    updated_at = datetime(2013, 1, 1, tzinfo=UTC)

    @factory.post_generation
    def locations(self, create, extracted, **kwargs):
        """The post_generation decorator performs actions once the model object has been generated."""
        if extracted is None:
            return

        if isinstance(extracted, str):
            extracted = [extracted]

        for group_name in extracted:
            self.locations.add(LocationFactory.simple_generate(create, name=group_name))

    @factory.post_generation
    def facilities(self, create, extracted, **kwargs):
        """The post_generation decorator performs actions once the model object has been generated."""
        if extracted is None:
            return

        if isinstance(extracted, str):
            extracted = [extracted]

        for group_name in extracted:
            self.facilities.add(FacilityFactory.simple_generate(create, name=group_name))

    @factory.post_generation
    def trip_schedule(self, create, number, **kwargs):
        """The post_generation decorator performs actions once the model object has been generated."""
        if number is None:
            return
        TripScheduleFactory.simple_generate_batch(create, trip=self, size=number)



# -*- coding: utf-8 -*-

from rest_framework.response import Response
from rest_framework.pagination import PageNumberPagination


class TripResponsePagination(PageNumberPagination):
    """
    API Custom paginator to tell number of pages
    on which data has been distributed.
    """

    page_size = 10

    def get_paginated_response(self, data):
        """
        Return custom paginated response as
        {
        'next': next page
        'previous': previous page
        'count': Total records
        'current': Current Page
        'pages': Total number of pages
        'data': data returned by API
        }
        """
        return Response({
            'next': self.get_next_link(),
            'previous': self.get_previous_link(),
            'count': self.page.paginator.count,
            'current': self.page.number,
            'pages': self.page.paginator.num_pages,
            'results': data,
        })



# -*- coding: utf-8 -*-
from datetime import datetime, timedelta
from django.utils import timezone
from django.test import TestCase
from pytz import UTC

from django_trips.models import (
    Facility,
    Host,
    Location,
    Trip,
    TripSchedule,
    CancellationPolicy,
)
from django_trips.tests.factories import (
    CategoryFactory,
    FacilityFactory,
    HostFactory,
    LocationFactory,
    TripFactory,
    TripItineraryFactory,
    TripScheduleFactory,
)


class TestHost(TestCase):
    """
    Test Suite for verifying various operations on Host model.
    """

    def setUp(self):
        self.host = HostFactory()

    def test_create_host(self):
        """
        Test that Host object creates successfully with required
        and optional fields.
        """
        # Successful Save will create id
        self.assertIsNotNone(self.host.id)
        self.assertIsNone(self.host.description)
        self.assertFalse(self.host.verified)

        # Adding Optional Fields
        self.host.description = "Travel Company"
        self.host.verified = True
        self.host.save()
        self.assertEqual(self.host.verified, True)
        self.assertIsNotNone(self.host.description)

    def test_update_host(self):
        """
        Test that updating the Host object works with changes
        reflected after the save
        """
        prev_description = self.host.description
        self.host.verified = True
        self.host.description = "Software Company"
        self.host.save()
        updated_host = Host.objects.filter(name=self.host.name)[0]
        self.assertNotEqual(prev_description, updated_host.description)

    def test_delete_host(self):
        """
        Test deleting an object works and object is not
        accessible after deletion
        """
        host_name = self.host.name
        self.host.delete()
        with self.assertRaises(Host.DoesNotExist):
            Host.objects.get(name=host_name)


class TestLocation(TestCase):
    """
    Test Suite to verify Location model w.r.t CRUD Operations.
    """

    def setUp(self):
        self.location = LocationFactory()

    def test_create_location(self):
        """
        Test location object is created successfully after save
        """
        self.assertIsNotNone(self.location.id)

    def test_update_location(self):
        """
        Test location objects are persisted after update.
        """
        self.assertIsNone(self.location.coordinates)
        self.location.coordinates = "-95, 45"
        self.location.save()
        updated_location = Location.objects.filter(name=self.location.name)[0]
        self.assertIsNotNone(updated_location.coordinates)

    def test_delete_location(self):
        """
        Test location objects are no longer persisted after delete.
        """
        location_name = self.location.name
        self.location.delete()
        with self.assertRaises(Location.DoesNotExist):
            Location.objects.get(name=location_name)


class TestFacility(TestCase):
    """
    Verify CRUD operations for Facility model.
    """

    def setUp(self):
        self.facility = FacilityFactory()

    def test_create_facility(self):
        """
        Test facility is created successfully after save operation.
        """
        self.assertIsNotNone(self.facility.id)

    def test_update_facility(self):
        """
        Test facility object updates are persisted successfully after save.
        """
        id = self.facility.id
        self.facility.name = "No Food"
        self.facility.save()
        updated_facility = Facility.objects.filter(id=id)[0]
        self.assertEqual("No Food", updated_facility.name)

    def test_delete_facility(self):
        """
        Test the facility object is not longer persisted after delete.
        """
        self.assertEqual(1, len(Facility.objects.all()))
        self.facility.delete()
        self.assertEqual(0, len(Facility.objects.all()))


class TestTrip(TestCase):
    """Test the application Trip model for CURD oprations"""

    def setUp(self):
        """Setup objects for testing"""
        self.trip = TripFactory.create(locations=["Lahore", "Gilgit"])

    def _update_trip_field(self, field, value):
        """Updates a field in trip object"""
        setattr(self.trip, field, value)
        self.trip.save()

    def test_create(self):
        """Checks if setup has created model object"""
        self.assertIsNotNone(self.trip.id)

    def test_update(self):
        """Test update trip method."""
        new_description = u"This is my dummy description"
        self._update_trip_field("description", new_description)

        trip = Trip.objects.get(id=self.trip.id)
        self.assertEqual(new_description, trip.description)

    def test_delete(self):
        """Test Trip delete"""
        self.assertEqual(Trip.objects.all().count(), 1)
        self.trip.delete()
        self.assertEqual(Trip.objects.all().count(), 0)

    def test_gear_update(self):
        """Test update gear"""
        new_gear = u"My test gear"
        self._update_trip_field("gear", new_gear)
        self.assertEqual(Trip.objects.get(id=self.trip.id).gear, new_gear)

    def test_trip_itinerary(self):
        """Test trip itinerary"""
        self.assertEqual(self.trip.trip_itinerary.all().count(), 0)
        TripItineraryFactory(trip=self.trip, day=1)
        TripItineraryFactory(trip=self.trip, day=2)
        self.assertEqual(self.trip.trip_itinerary.all().count(), 2)

    def test_trip_schedule(self):
        """Test Trip schedule"""
        today_date = datetime.now(tz=UTC)
        self.assertEqual(self.trip.trip_schedule.all().count(), 0)
        __ = TripScheduleFactory(trip=self.trip, date_from=today_date)
        trip_schedules = self.trip.trip_schedule.all()
        self.assertEqual(trip_schedules.all().count(), 1)
        self.assertEqual(trip_schedules.get().date_from, today_date)

    def test_trip_availability(self):
        """Test available manager of trip schedule"""
        future_trip_date = timezone.now() + timedelta(days=7)
        past_trip_date = timezone.now() - timedelta(days=7)
        __ = TripScheduleFactory(trip=self.trip, date_from=past_trip_date)

        __ = TripScheduleFactory(trip=self.trip, date_from=future_trip_date)
        self.assertEqual(TripSchedule.available.all().count(), 1)

    def test_cancellation_policy(self):
        """
        Tests cancellation policy override.

        When trip object has cancellation policy, it should be given perfernce
        over host cancellation policy.
        """
        self.trip.host.cancellation_policy = None
        self.trip.host.save()

        common_policy = "Common trip CancellationPolicy"
        CancellationPolicy(description=common_policy).save()
        new_cancellation_policy = "new cancellation policy"

        self.assertEqual(self.trip.cancellation_policy, CancellationPolicy.current().description)

        # Update host policy.
        self.trip.host.cancellation_policy = new_cancellation_policy
        self.trip.host.save()
        self.assertEqual(self.trip.cancellation_policy, new_cancellation_policy)
